@@ -553,7 +553,11 @@ int cloudi_initialize(cloudi_instance_t * api,
         return cloudi_out_of_memory;
     char const * const protocol = ::getenv("CLOUDI_API_INIT_PROTOCOL");
     if (protocol == 0)
+    {
+        std::cerr <<
+            "CloudI service execution must occur in CloudI" << std::endl;
         return cloudi_invalid_input;
+    }
     char const * const buffer_size_p = ::getenv("CLOUDI_API_INIT_BUFFER_SIZE");
     if (buffer_size_p == 0)
         return cloudi_invalid_input;
@@ -1409,6 +1413,11 @@ static void callback(cloudi_instance_t * api,
             std::cerr << boost::diagnostic_information(e);
             return;
         }
+        catch (CloudI::API::fatal_error const & e)
+        {
+            std::cerr << boost::diagnostic_information(e);
+            ::exit(1);
+        }
         catch (boost::exception const & e)
         {
             std::cerr << boost::diagnostic_information(e);
@@ -1462,6 +1471,11 @@ static void callback(cloudi_instance_t * api,
             std::cerr << boost::diagnostic_information(e);
             return;
         }
+        catch (CloudI::API::fatal_error const & e)
+        {
+            std::cerr << boost::diagnostic_information(e);
+            ::exit(1);
+        }
         catch (boost::exception const & e)
         {
             std::cerr << boost::diagnostic_information(e);
@@ -1501,7 +1515,7 @@ static void store_incoming_binary(buffer_t const & buffer,
     uint32_t size = *reinterpret_cast<uint32_t *>(&buffer[index]);
     index += sizeof(uint32_t);
     p = new char[size];
-    memcpy(p, &buffer[index], size);
+    ::memcpy(p, &buffer[index], size);
     index += size;
 }
 
@@ -1888,10 +1902,10 @@ static char const ** text_pairs_parse(void const * const text,
 {
     char const * text_p = reinterpret_cast<char const *>(text);
     realloc_ptr<char const *> pairs(16, 268435456);
-    size_t i = 1;
-    pairs[0] = text_p;
     if (text_size > 1)
     {
+        size_t i = 1;
+        pairs[0] = text_p;
         for (size_t text_i = 1; text_i < text_size - 1; ++text_i)
         {
             if (text_p[text_i] == '\0')
@@ -1901,8 +1915,13 @@ static char const ** text_pairs_parse(void const * const text,
                     break;
             }
         }
+        pairs[i] = 0;
+    }
+    else
+    {
+        pairs[0] = 0;
+        pairs[1] = 0;
     }
-    pairs[i] = 0;
     return pairs.release(true);
 }
 
