@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -42,6 +42,7 @@ module Foreign.CloudI
     , messageDecodingError
     , terminateError
     , Exception
+    , FatalError
     , Result
     , api
     , threadCount
@@ -72,9 +73,10 @@ module Foreign.CloudI
     , threadCreate
     , threadsWait
     , infoKeyValueParse
+    , infoKeyValueNew
     ) where
 
-import Prelude hiding (init,length)
+import Prelude hiding (init,length,(<>))
 import Data.Bits (shiftL,(.|.))
 import Data.Maybe (fromMaybe)
 import Data.Typeable (Typeable)
@@ -97,8 +99,11 @@ import qualified Foreign.C.Types as C
 import qualified Foreign.Erlang as Erlang
 import qualified Foreign.CloudI.Instance as Instance
 import qualified System.IO as SysIO
+import qualified System.IO.Error as SysIOErr
 import qualified System.IO.Unsafe as Unsafe
 import qualified System.Posix.Env as POSIX (getEnv)
+import qualified System.Posix.Process as POSIX (exitImmediately)
+import qualified System.Exit as Exit
 type Array = IArray.Array
 type Builder = Builder.Builder
 type ByteString = ByteString.ByteString
@@ -108,6 +113,10 @@ type LazyByteString = LazyByteString.ByteString
 type Map = Map.Map
 type RequestType = Instance.RequestType
 type SomeException = Exception.SomeException
+type SomeAsyncException = Exception.SomeAsyncException
+type AsyncException = Exception.AsyncException
+type AssertionFailed = Exception.AssertionFailed
+type ExitCode = Exit.ExitCode
 type Source = Instance.Source
 type ThreadId = Concurrent.ThreadId
 type Word32 = Word.Word32
@@ -158,10 +167,18 @@ data Exception s =
     | ReturnAsync (Instance.T s)
     | ForwardSync (Instance.T s)
     | ForwardAsync (Instance.T s)
+    | Terminate (Instance.T s)
     deriving (Show, Typeable)
 
 instance Typeable s => Exception.Exception (Exception s)
 
+data FatalError = FatalError
+
+instance Exception.Exception FatalError
+
+instance Show FatalError where
+    showsPrec _ FatalError = showString "FatalError"
+
 printException :: String -> IO ()
 printException str =
     SysIO.hPutStrLn SysIO.stderr ("Exception: " ++ str)
@@ -178,10 +195,14 @@ data CallbackResult s =
 
 type Result a = Either String a
 
+infixr 4 <>
+(<>) :: Monoid.Monoid m => m -> m -> m
+(<>) = Monoid.mappend
+
 -- | creates an instance of the CloudI API
-api :: Typeable s => Int -> s ->
+api :: Typeable s => Int -> s -> Maybe Bool ->
     IO (Result (Instance.T s))
-api threadIndex state = do
+api threadIndex state terminateReturnValueOpt = do
     SysIO.hSetEncoding SysIO.stdout SysIO.utf8
     SysIO.hSetBuffering SysIO.stdout SysIO.LineBuffering
     SysIO.hSetEncoding SysIO.stderr SysIO.utf8
@@ -190,26 +211,31 @@ api threadIndex state = do
     bufferSizeValue <- POSIX.getEnv "CLOUDI_API_INIT_BUFFER_SIZE"
     case (protocolValue, bufferSizeValue) of
         (Just protocol, Just bufferSizeStr) ->
-            let bufferSize = read bufferSizeStr :: Int
+            let terminateReturnValue = fromMaybe True terminateReturnValueOpt
+                terminateException = not terminateReturnValue
+                bufferSize = read bufferSizeStr :: Int
                 fd = C.CInt $ fromIntegral (threadIndex + 3)
                 useHeader = protocol /= "udp"
-                timeoutTerminate' = 1000
+                timeoutTerminate' = 10 -- TIMEOUT_TERMINATE_MIN
                 initTerms = Erlang.OtpErlangAtom (Char8.pack "init")
             in
             case Erlang.termToBinary initTerms (-1) of
                 Left err ->
                     return $ Left $ show err
                 Right initBinary -> do
-                    api0 <- Instance.make state protocol fd
-                        useHeader bufferSize timeoutTerminate'
+                    api0 <- Instance.make state terminateException
+                        protocol fd useHeader bufferSize timeoutTerminate'
                     send api0 initBinary
                     result <- pollRequest api0 (-1) False
                     case result of
                         Left err ->
+                            -- Terminate exception not used here
                             return $ Left err
                         Right (_, api1) ->
                             return $ Right api1
-        (_, _) ->
+        (_, _) -> do
+            SysIO.hPutStrLn SysIO.stderr
+                "CloudI service execution must occur in CloudI"
             return $ Left invalidInputError
 
 -- | returns the thread count from the service configuration
@@ -240,7 +266,9 @@ subscribe api0 pattern f =
 -- | returns the number of subscriptions for a single service name pattern
 subscribeCount :: Typeable s => Instance.T s -> ByteString ->
     IO (Result (Int, Instance.T s))
-subscribeCount api0 pattern =
+subscribeCount api0@Instance.T{
+      Instance.terminateException = terminateException}
+    pattern =
     let subscribeCountTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "subscribe_count")
             , Erlang.OtpErlangString pattern]
@@ -253,7 +281,10 @@ subscribeCount api0 pattern =
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.subscribeCount = count}) ->
                     return $ Right (count, api1)
 
@@ -277,7 +308,8 @@ sendAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (ByteString, Instance.T s))
 sendAsync api0@Instance.T{
-      Instance.timeoutAsync = timeoutAsync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutAsync = timeoutAsync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
@@ -299,7 +331,10 @@ sendAsync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.transId = transId}) ->
                     return $ Right (transId, api1)
 
@@ -308,7 +343,8 @@ sendSync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (ByteString, ByteString, ByteString, Instance.T s))
 sendSync api0@Instance.T{
-      Instance.timeoutSync = timeoutSync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutSync = timeoutSync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutSync' timeoutOpt
@@ -330,7 +366,10 @@ sendSync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{
                       Instance.responseInfo = responseInfo
                     , Instance.response = response
@@ -343,7 +382,8 @@ mcastAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     Maybe Int -> Maybe ByteString -> Maybe Int ->
     IO (Result (Array Int ByteString, Instance.T s))
 mcastAsync api0@Instance.T{
-      Instance.timeoutAsync = timeoutAsync'
+      Instance.terminateException = terminateException
+    , Instance.timeoutAsync = timeoutAsync'
     , Instance.priorityDefault = priorityDefault}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
@@ -365,7 +405,10 @@ mcastAsync api0@Instance.T{
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{Instance.transIds = transIds}) ->
                     return $ Right (transIds, api1)
 
@@ -521,7 +564,9 @@ returnSync api0 name pattern responseInfo response timeout transId pid = do
 recvAsync :: Typeable s => Instance.T s ->
     Maybe Int -> Maybe ByteString -> Maybe Bool ->
     IO (Result (ByteString, ByteString, ByteString, Instance.T s))
-recvAsync api0@Instance.T{Instance.timeoutSync = timeoutSync'}
+recvAsync api0@Instance.T{
+      Instance.terminateException = terminateException
+    , Instance.timeoutSync = timeoutSync'}
     timeoutOpt transIdOpt consumeOpt =
     let timeout = fromMaybe timeoutSync' timeoutOpt
         transId = fromMaybe transIdNull transIdOpt
@@ -540,7 +585,10 @@ recvAsync api0@Instance.T{Instance.timeoutSync = timeoutSync'}
             result <- pollRequest api0 (-1) False
             case result of
                 Left err ->
-                    return $ Left err
+                    if err == terminateError && terminateException then
+                        Exception.throwIO $ Terminate api0
+                    else
+                        return $ Left err
                 Right (_, api1@Instance.T{
                       Instance.responseInfo = responseInfo
                     , Instance.response = response
@@ -604,6 +652,26 @@ nullResponse :: RequestType -> ByteString -> ByteString ->
 nullResponse _ _ _ _ _ _ _ _ _ state api0 =
     return $ Instance.Null (state, api0)
 
+callbackExceptionFatal :: SomeException -> IO ()
+callbackExceptionFatal e = do
+    printException $ show e
+    POSIX.exitImmediately (Exit.ExitFailure 1)
+
+callbackException :: SomeException -> IO ()
+callbackException e
+    | Just _ <- (Exception.fromException e :: Maybe AsyncException) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe SomeAsyncException) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe AssertionFailed) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe FatalError) =
+        callbackExceptionFatal e
+    | Just exitCode <- (Exception.fromException e :: Maybe ExitCode) =
+        POSIX.exitImmediately exitCode
+    | otherwise =
+        printException $ show e
+
 callback :: Typeable s => Instance.T s ->
     (RequestType, ByteString, ByteString, ByteString, ByteString,
      Int, Int, ByteString, Source) -> IO (Result (Instance.T s))
@@ -618,7 +686,7 @@ callback api0@Instance.T{
             Just functionQueue ->
                 let f = Sequence.index functionQueue 0
                     functionQueueNew = (Sequence.|>)
-                        (Sequence.drop 0 functionQueue) f
+                        (Sequence.drop 1 functionQueue) f
                 in
                 (f, Map.insert pattern functionQueueNew callbacks)
         api1 = api0{Instance.callbacks = callbacksNew}
@@ -630,16 +698,18 @@ callback api0@Instance.T{
                 requestInfo request timeout priority transId pid
                 state api1
             case callbackResultAsyncValue of
+                Left (Terminate api2) ->
+                    return $ ReturnI (empty, empty, state, setTerminate api2)
+                Left (ReturnAsync api2) ->
+                    return $ Finished api2
                 Left (ReturnSync api2) -> do
                     printException "Synchronous Call Return Invalid"
-                    return $ Finished api2
-                Left (ReturnAsync api2) ->
+                    return $ Finished (setTerminate api2)
+                Left (ForwardAsync api2) ->
                     return $ Finished api2
                 Left (ForwardSync api2) -> do
                     printException "Synchronous Call Forward Invalid"
-                    return $ Finished api2
-                Left (ForwardAsync api2) ->
-                    return $ Finished api2
+                    return $ Finished (setTerminate api2)
                 Right (Instance.ResponseInfo (v0, v1, v2, v3)) ->
                     return $ ReturnI (v0, v1, v2, v3)
                 Right (Instance.Response (v0, v1, v2)) ->
@@ -659,16 +729,18 @@ callback api0@Instance.T{
                 requestInfo request timeout priority transId pid
                 state api1
             case callbackResultSyncValue of
+                Left (Terminate api2) ->
+                    return $ ReturnI (empty, empty, state, setTerminate api2)
                 Left (ReturnSync api2) ->
                     return $ Finished api2
                 Left (ReturnAsync api2) -> do
                     printException "Asynchronous Call Return Invalid"
-                    return $ Finished api2
+                    return $ Finished (setTerminate api2)
                 Left (ForwardSync api2) ->
                     return $ Finished api2
                 Left (ForwardAsync api2) -> do
                     printException "Asynchronous Call Forward Invalid"
-                    return $ Finished api2
+                    return $ Finished (setTerminate api2)
                 Right (Instance.ResponseInfo (v0, v1, v2, v3)) ->
                     return $ ReturnI (v0, v1, v2, v3)
                 Right (Instance.Response (v0, v1, v2)) ->
@@ -684,8 +756,8 @@ callback api0@Instance.T{
                     return $ ReturnI (empty, empty, v0, v1)
     callbackResultType <- case callbackResultValue of
         Left exception -> do
-            printException $ show (exception :: Exception.SomeException)
-            return $ Finished api1
+            callbackException exception
+            return $ ReturnI (empty, empty, state, api1)
         Right callbackResult ->
             return $ callbackResult
     case requestType of
@@ -720,11 +792,8 @@ handleEvents messages api0 external cmd0 = do
     cmd <- if cmd0 == 0 then Get.getWord32host else return cmd0
     case () of
       _ | cmd == messageTerm ->
-            let api1 = api0{
-                      Instance.terminate = True
-                    , Instance.timeout = Just False} in
             if external then
-                return ([], api1)
+                return ([], setTerminate api0)
             else
                 fail terminateError
         | cmd == messageReinit -> do
@@ -928,16 +997,15 @@ pollRequestData api0 external dataIn =
 pollRequestLoop :: Typeable s =>
     Instance.T s -> Int -> Bool -> Clock.NominalDiffTime ->
     IO (Result (Bool, Instance.T s))
-pollRequestLoop api0@Instance.T{
-      Instance.socketHandle = socketHandle
-    , Instance.bufferRecvSize = bufferRecvSize}
-    timeout external pollTimer = do
-    inputAvailable <- if bufferRecvSize > 0 then
-            return True
-        else
-            SysIO.hWaitForInput socketHandle timeout
-    if not inputAvailable then
+pollRequestLoop api0 timeout external pollTimer = do
+    inputAvailable <- pollWait api0 timeout
+    if inputAvailable == Just False then
         return $ Right (True, api0{Instance.timeout = Nothing})
+    else if inputAvailable == Nothing then
+        let api1 = api0{
+              Instance.terminate = True
+            , Instance.timeout = Nothing} in
+        return $ Right (False, api1)
     else do
         (dataIn, _, api1) <- recv api0
         dataResult <- pollRequestData api1 external dataIn
@@ -973,7 +1041,10 @@ pollRequest api0@Instance.T{
       Instance.initializationComplete = initializationComplete
     , Instance.terminate = terminate} timeout external =
     if terminate then
-        return $ Right (False, api0)
+        if external then
+            return $ Right (False, api0)
+        else
+            return $ Left terminateError
     else do
         pollTimer <- if timeout <= 0 then
                 return 0
@@ -992,6 +1063,23 @@ pollRequest api0@Instance.T{
         else
             pollRequestLoopBegin api0 timeout external pollTimer
 
+pollWait :: Typeable s => Instance.T s -> Int -> IO (Maybe Bool)
+pollWait Instance.T{
+      Instance.socketHandle = socketHandle
+    , Instance.bufferRecvSize = bufferRecvSize}
+    timeout = do
+    if bufferRecvSize > 0 then
+        return $ Just True
+    else
+        SysIOErr.catchIOError
+            (do
+             inputAvailable <- SysIO.hWaitForInput socketHandle timeout
+             return $ Just inputAvailable)
+            (\e -> if SysIOErr.isEOFError e then
+                 return Nothing
+             else
+                 SysIOErr.ioError e)
+
 -- | blocks to process incoming CloudI service requests
 poll :: Typeable s => Instance.T s -> Int -> IO (Result (Bool, Instance.T s))
 poll api0 timeout =
@@ -1111,6 +1199,11 @@ timeoutAdjustmentPoll t0 timeout = do
         else
             return (t1, fromIntegral $ timeoutValue - elapsed)
 
+setTerminate :: Instance.T s -> Instance.T s
+setTerminate api0 =
+    api0{ Instance.terminate = True
+        , Instance.timeout = Just False}
+
 threadList :: Concurrent.MVar [Concurrent.MVar ()]
 threadList = Unsafe.unsafePerformIO (Concurrent.newMVar [])
 
@@ -1147,8 +1240,8 @@ threadsWait = do
             Concurrent.takeMVar done
             threadsWait
 
-textKeyValueParse :: ByteString -> Map ByteString [ByteString]
-textKeyValueParse text =
+textPairsParse :: ByteString -> Map ByteString [ByteString]
+textPairsParse text =
     let loop m [] = m
         loop m [v] =
             if v == ByteString.empty then
@@ -1160,11 +1253,31 @@ textKeyValueParse text =
                 Nothing ->
                     loop (Map.insert k [v] m) l'
                 Just v' ->
-                    loop (Map.insert k (v:v') m) l'
+                    loop (Map.insert k (v' ++ [v]) m) l'
     in
     loop Map.empty (Char8.split '\0' text)
 
--- | parses "text_pairs" in service request info
+textPairsNew :: Map ByteString [ByteString] -> Maybe Bool -> ByteString
+textPairsNew pairs responseOpt =
+    let response = fromMaybe True responseOpt in
+    if response && Map.size pairs == 0 then
+        Char8.pack "\0"
+    else
+        let pair builder _ [] =
+                builder
+            pair builder k (v:l') =
+                pair (builder <>
+                    Builder.byteString k <> Builder.char8 '\0' <>
+                    Builder.byteString v <> Builder.char8 '\0') k l'
+        in
+        LazyByteString.toStrict $ Builder.toLazyByteString $
+            Map.foldlWithKey pair Monoid.mempty pairs
+
+-- | decode service request info key/value data
 infoKeyValueParse :: ByteString -> Map ByteString [ByteString]
-infoKeyValueParse = textKeyValueParse
+infoKeyValueParse = textPairsParse
+
+-- | encode service response info key/value data
+infoKeyValueNew :: Map ByteString [ByteString] -> Maybe Bool -> ByteString
+infoKeyValueNew = textPairsNew
 
