@@ -3,8 +3,6 @@
 %%% This file is part of cloudi_x_hackney released under the Apache 2 license.
 %%% See the NOTICE for more information.
 %%%
-%%% Copyright (c) 2012-2014 Benoît Chesneau <benoitc@e-engura.org>
-%%%
 
 -module(cloudi_x_hackney_sup).
 
@@ -16,6 +14,8 @@
 %% Supervisor callbacks
 -export([init/1]).
 
+-include("cloudi_x_hackney.hrl").
+
 %% Helper macro for declaring children of supervisor
 -define(CHILD(I, Type), {I, {I, start_link, []}, permanent, 5000, Type, [I]}).
 
@@ -24,20 +24,30 @@
 %% ===================================================================
 
 start_link() ->
-    {ok, Pid} = supervisor:start_link({local, ?MODULE}, ?MODULE, []),
-    %% start the pool handler
-    PoolHandler = cloudi_x_hackney_app:get_app_env(pool_handler, cloudi_x_hackney_pool),
-    ok = PoolHandler:start(),
-
-    %% finish to start the application
-    {ok, Pid}.
+  {ok, Pid} = supervisor:start_link({local, ?MODULE}, ?MODULE, []),
+  %% start the pool handler
+  PoolHandler = cloudi_x_hackney_app:get_app_env(pool_handler, cloudi_x_hackney_pool),
+  ok = PoolHandler:start(),
+  %% finish to start the application
+  {ok, Pid}.
 
 %% ===================================================================
 %% Supervisor callbacks
 %% ===================================================================
 
 init([]) ->
+  %% initialize the config table
+  _ = ets:new(?CONFIG, [set, named_table, public]),
 
-    Manager = ?CHILD(cloudi_x_hackney_manager, worker),
+  %% initialize the metric engine
+  cloudi_x_hackney_metrics:init(),
+
+  Specs = [
+           %% manager
+           ?CHILD(cloudi_x_hackney_manager, worker),
+           %% connections manager
+           ?CHILD(cloudi_x_hackney_connections, worker)
+          ],
+
+  {ok, { {one_for_one, 10000, 1}, Specs}}.
 
-    {ok, { {one_for_one, 10000, 1}, [Manager]}}.
