@@ -41,43 +41,12 @@
     return_maps = false
 }).
 
--type config() :: list().
--export_type([config/0]).
+-type config() :: proplists:proplist().
 
--ifndef(maps_support).
--type json_value() :: list(json_value())
-    | list({binary() | atom(), json_value()}) | [{},...]
-    | true
-    | false
-    | null
-    | integer()
-    | float()
-    | binary().
--endif.
+-spec to_term(Source::binary(), Config::cloudi_x_jsx_config:options()) -> cloudi_x_jsx:json_term() | {incomplete, cloudi_x_jsx:decoder()}.
 
--ifdef(maps_support).
--type json_value() :: list(json_value())
-    | list({binary() | atom(), json_value()}) | [{},...]
-    | map()
-    | true
-    | false
-    | null
-    | integer()
-    | float()
-    | binary().
--endif.
-
-
--spec to_term(Source::binary(), Config::config()) -> json_value().
-
--ifdef(maps_always).
 to_term(Source, Config) when is_list(Config) ->
     (cloudi_x_jsx:decoder(?MODULE, [return_maps] ++ Config, cloudi_x_jsx_config:extract_config(Config)))(Source).
--endif.
--ifndef(maps_always).
-to_term(Source, Config) when is_list(Config) ->
-    (cloudi_x_jsx:decoder(?MODULE, Config, cloudi_x_jsx_config:extract_config(Config)))(Source).
--endif.
 
 parse_config(Config) -> parse_config(Config, #config{}).
 
@@ -86,7 +55,8 @@ parse_config([{labels, Val}|Rest], Config)
     parse_config(Rest, Config#config{labels = Val});
 parse_config([labels|Rest], Config) ->
     parse_config(Rest, Config#config{labels = binary});
-parse_config([{return_maps, Val}|Rest], Config) when is_boolean(Val) ->
+parse_config([{return_maps, Val}|Rest], Config)
+        when Val == true; Val == false ->
     parse_config(Rest, Config#config{return_maps = Val});
 parse_config([return_maps|Rest], Config) ->
     parse_config(Rest, Config#config{return_maps = true});
@@ -105,7 +75,7 @@ parse_config([], Config) ->
 
 
 -type state() :: {list(), #config{}}.
--spec init(Config::proplists:proplist()) -> state().
+-spec init(Config::config()) -> state().
 
 init(Config) -> start_term(Config).
 
@@ -163,41 +133,6 @@ format_key(Key, Config) ->
 
 start_term(Config) when is_list(Config) -> {[], parse_config(Config)}.
 
-
--ifndef(maps_support).
-%% allocate a new object on top of the stack
-start_object({Stack, Config}) -> {[{object, []}] ++ Stack, Config}.
-
-
-%% allocate a new array on top of the stack
-start_array({Stack, Config}) -> {[{array, []}] ++ Stack, Config}.
-
-
-%% finish an object or array and insert it into the parent object if it exists or
-%% return it if it is the root object
-finish({[{object, []}], Config}) -> {[{}], Config};
-finish({[{object, []}|Rest], Config}) -> insert([{}], {Rest, Config});
-finish({[{object, Pairs}], Config}) -> {lists:reverse(Pairs), Config};
-finish({[{object, Pairs}|Rest], Config}) -> insert(lists:reverse(Pairs), {Rest, Config});
-finish({[{array, Values}], Config}) -> {lists:reverse(Values), Config};
-finish({[{array, Values}|Rest], Config}) -> insert(lists:reverse(Values), {Rest, Config});
-finish(_) -> erlang:error(badarg).
-
-
-%% insert a value when there's no parent object or array
-insert(Value, {[], Config}) -> {Value, Config};
-%% insert a key or value into an object or array, autodetects the 'right' thing
-insert(Key, {[{object, Pairs}|Rest], Config}) ->
-    {[{object, Key, Pairs}] ++ Rest, Config};
-insert(Value, {[{object, Key, Pairs}|Rest], Config}) ->
-    {[{object, [{Key, Value}] ++ Pairs}] ++ Rest, Config};
-insert(Value, {[{array, Values}|Rest], Config}) ->
-    {[{array, [Value] ++ Values}] ++ Rest, Config};
-insert(_, _) -> erlang:error(badarg).
--endif.
-
-
--ifdef(maps_support).
 %% allocate a new object on top of the stack
 start_object({Stack, Config=#config{return_maps=true}}) ->
     {[{object, #{}}] ++ Stack, Config};
@@ -236,8 +171,6 @@ insert(Value, {[{object, Key, Pairs}|Rest], Config}) ->
 insert(Value, {[{array, Values}|Rest], Config}) ->
     {[{array, [Value] ++ Values}] ++ Rest, Config};
 insert(_, _) -> erlang:error(badarg).
--endif.
-
 
 get_key({[{object, Key, _}|_], _}) -> Key;
 get_key(_) -> erlang:error(badarg).
@@ -365,7 +298,6 @@ rep_manipulation_test_() ->
     ].
 
 
--ifdef(maps_support).
 rep_manipulation_with_maps_test_() ->
     [
         {"allocate a new object on an empty stack", ?_assertEqual(
@@ -417,10 +349,10 @@ return_maps_test_() ->
     [
         {"an empty map", ?_assertEqual(
             #{},
-            cloudi_x_jsx:decode(<<"{}">>, [return_maps])
+            cloudi_x_jsx:decode(<<"{}">>, [])
         )},
         {"an empty map", ?_assertEqual(
-            [{}],
+            #{},
             cloudi_x_jsx:decode(<<"{}">>, [])
         )},
         {"an empty map", ?_assertEqual(
@@ -429,18 +361,17 @@ return_maps_test_() ->
         )},
         {"a small map", ?_assertEqual(
             #{<<"awesome">> => true, <<"library">> => <<"cloudi_x_jsx">>},
-            cloudi_x_jsx:decode(<<"{\"library\": \"cloudi_x_jsx\", \"awesome\": true}">>, [return_maps])
+            cloudi_x_jsx:decode(<<"{\"library\": \"cloudi_x_jsx\", \"awesome\": true}">>, [])
         )},
         {"a recursive map", ?_assertEqual(
             #{<<"key">> => #{<<"key">> => true}},
-            cloudi_x_jsx:decode(<<"{\"key\": {\"key\": true}}">>, [return_maps])
+            cloudi_x_jsx:decode(<<"{\"key\": {\"key\": true}}">>, [])
         )},
         {"a map inside a list", ?_assertEqual(
             [#{}],
-            cloudi_x_jsx:decode(<<"[{}]">>, [return_maps])
+            cloudi_x_jsx:decode(<<"[{}]">>, [])
         )}
     ].
--endif.
 
 
 handle_event_test_() ->
