@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2016-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2016-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2016-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @copyright 2016-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_cgroups).
@@ -45,7 +45,8 @@
          new/1,
          shell/2,
          update/4,
-         update_or_create/4]).
+         update_or_create/4,
+         version/1]).
 
 -record(cloudi_x_cgroups,
     {
@@ -75,8 +76,9 @@
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Create a specific cgroup.===
-%% Files cpuset.cpus and cpuset.mems are set if they are not initialized
-%% due to cgroup.clone_children (using the root values).
+%% With cloudi_x_cgroups v1, files cpuset.cpus and cpuset.mems are set
+%% if they are not initialized due to cgroup.clone_children
+%% (using the root values).
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -185,9 +187,15 @@ destroy(#cloudi_x_cgroups{path = Path,
 %% @end
 %%-------------------------------------------------------------------------
 
+-type new_error_reasons() ::
+    {'version_default', Version :: pos_integer()} |
+    {'path_mounts', Status :: pos_integer(), Output :: list(binary())} |
+    {'path_v1', Status :: pos_integer(), Output :: list(binary())} |
+    {'path_v2', Status :: pos_integer(), Output :: list(binary())}.
+
 -spec new() ->
     {ok, #cloudi_x_cgroups{}} |
-    {error, any()}.
+    {error, new_error_reasons()}.
 
 new() ->
     new([]).
@@ -200,7 +208,7 @@ new() ->
 
 -spec new(Options0 :: options()) ->
     {ok, #cloudi_x_cgroups{}} |
-    {error, any()}.
+    {error, new_error_reasons()}.
 
 new(Options0) ->
     {VersionDefault,
@@ -240,7 +248,7 @@ shell(Command, Arguments) ->
                              [{args, ["-"]}, {cd, "/"},
                               stream, binary, stderr_to_stdout, exit_status]),
     Exec = io_lib:format(Command, Arguments),
-    true = erlang:port_command(Shell, ["exec ", Exec, "\n"]),
+    true = erlang:port_command(Shell, [Exec, "\nexit $?\n"]),
     shell_output(Shell, []).
 
 %%-------------------------------------------------------------------------
@@ -281,10 +289,10 @@ update(CGroupPath, OSPids, CGroupParameters,
             {error, {invalid_cgroup_parameters, CGroupParameters}};
         true ->
             CGroupPathFull = Path ++ CGroupPath,
-            case update_parameters(CGroupParameters, Version,
+            case update_parameters(Version, CGroupParameters,
                                    CGroupPathFull, Path) of
                 ok ->
-                    update_pids(OSPids, Version, CGroupPathFull);
+                    update_pids(OSPids, CGroupPathFull);
                 {error, _} = Error ->
                     Error
             end
@@ -319,6 +327,18 @@ update_or_create([_ | _] = CGroupPath, OSPids, CGroupParameters,
             end
     end.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===The cgroup version used.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec version(#cloudi_x_cgroups{}) ->
+    pos_integer().
+
+version(#cloudi_x_cgroups{version = Version}) ->
+    Version.
+
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
@@ -430,11 +450,12 @@ new_mounts([Mount | MountsL], PathV1, PathV2) ->
     end.
 
 create_cgroup(CGroupPath, OSPids, CGroupParameters,
-              #cloudi_x_cgroups{path = Path} = State) ->
+              #cloudi_x_cgroups{version = Version,
+                       path = Path} = State) ->
     CGroupPathFull = Path ++ CGroupPath,
     case shell("mkdir -p \"~s\"", [CGroupPathFull]) of
         {0, _} ->
-            case create_update(CGroupPathFull, Path) of
+            case create_update(Version, CGroupPathFull, Path) of
                 ok ->
                     update(CGroupPath, OSPids, CGroupParameters, State);
                 {error, _} = Error ->
@@ -444,34 +465,37 @@ create_cgroup(CGroupPath, OSPids, CGroupParameters,
             {error, {mkdir, Status, Output}}
     end.
 
-create_update(CGroupPathFull, Path) ->
-    case create_update_get(CGroupPathFull, "cpuset.cpus", Path) of
+create_update(1, CGroupPathFull, Path) ->
+    case create_update_v1_get(CGroupPathFull, "cpuset.cpus", Path) of
         {ok, CPUS} ->
-            case create_update_get(CGroupPathFull, "cpuset.mems", Path) of
+            case create_update_v1_get(CGroupPathFull, "cpuset.mems", Path) of
                 {ok, MEMS} ->
                     if
                         CPUS =:= undefined,
                         MEMS =:= undefined ->
                             ok;
                         true ->
-                            create_update_set(CGroupPathFull, CPUS, MEMS, Path)
+                            create_update_v1_set(CGroupPathFull,
+                                                 CPUS, MEMS, Path)
                     end;
                 {error, _} = Error ->
                     Error
             end;
         {error, _} = Error ->
             Error
-    end.
+    end;
+create_update(2, _, _) ->
+    ok.
 
-create_update_set(CGroupPathFull, CPUS, MEMS, Path) ->
-    case create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) of
+create_update_v1_set(CGroupPathFull, CPUS, MEMS, Path) ->
+    case create_update_v1_set_subpath(CGroupPathFull, CPUS, MEMS, Path) of
         ok ->
             CGroupSubPathFull = CGroupPathFull ++ "/",
-            case create_update_set_value(CGroupSubPathFull,
-                                         "cpuset.cpus", CPUS) of
+            case create_update_v1_set_value(CGroupSubPathFull,
+                                            "cpuset.cpus", CPUS) of
                 ok ->
-                    case create_update_set_value(CGroupSubPathFull,
-                                                 "cpuset.mems", MEMS) of
+                    case create_update_v1_set_value(CGroupSubPathFull,
+                                                    "cpuset.mems", MEMS) of
                         ok ->
                             ok;
                         {error, _} = Error ->
@@ -484,19 +508,20 @@ create_update_set(CGroupPathFull, CPUS, MEMS, Path) ->
             Error
     end.
 
-create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
+create_update_v1_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
     case subdirectory(CGroupPathFull) of
         Path ->
             ok;
         CGroupSubPathFull ->
-            case create_update_set_subpath(CGroupSubPathFull,
-                                           CPUS, MEMS, Path) of
+            case create_update_v1_set_subpath(CGroupSubPathFull,
+                                              CPUS, MEMS, Path) of
                 ok ->
-                    case create_update_set_value(CGroupSubPathFull,
-                                                 "cpuset.cpus", CPUS) of
+                    case create_update_v1_set_value(CGroupSubPathFull,
+                                                    "cpuset.cpus", CPUS) of
                         ok ->
-                            case create_update_set_value(CGroupSubPathFull,
-                                                         "cpuset.mems", MEMS) of
+                            case create_update_v1_set_value(CGroupSubPathFull,
+                                                            "cpuset.mems",
+                                                            MEMS) of
                                 ok ->
                                     ok;
                                 {error, _} = Error ->
@@ -510,9 +535,9 @@ create_update_set_subpath(CGroupPathFull, CPUS, MEMS, Path) ->
             end
     end.
 
-create_update_set_value(_, _, undefined) ->
+create_update_v1_set_value(_, _, undefined) ->
     ok;
-create_update_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
+create_update_v1_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
     case shell("cat \"~s~s\"", [CGroupSubPathFull, SubsystemParameter]) of
         {0, OldValue} ->
             NeedsUpdate = strip(shell_output_string(OldValue)) == "",
@@ -537,7 +562,7 @@ create_update_set_value(CGroupSubPathFull, SubsystemParameter, Value) ->
             {error, {ErrorReason, Status, Output}}
     end.
 
-create_update_get(CGroupPathFull, SubsystemParameter, Path) ->
+create_update_v1_get(CGroupPathFull, SubsystemParameter, Path) ->
     case shell("cat \"~s/~s\"", [CGroupPathFull, SubsystemParameter]) of
         {0, Contents} ->
             NeedsUpdate = strip(shell_output_string(Contents)) == "",
@@ -564,32 +589,17 @@ delete_recursive_subpath(CGroupSubPathFull, Path) ->
             ok
     end.
 
-update_parameters(CGroupParameters, 1, CGroupPathFull, _) ->
+update_parameters(1, CGroupParameters, CGroupPathFull, _) ->
     update_parameters(CGroupParameters, CGroupPathFull);
-update_parameters(CGroupParameters, 2, CGroupPathFull, Path) ->
+update_parameters(2, CGroupParameters, CGroupPathFull, Path) ->
     Controllers = lists:usort([subsystem(SubsystemParameter)
                                || {SubsystemParameter, _} <- CGroupParameters]),
     ControlAdded = ["+" ++ Controller || Controller <- Controllers],
-    ControlRemoved = ["-" ++ Controller || Controller <- Controllers],
-    CGroupSubPathFull = subdirectory(CGroupPathFull),
-    case subtree_control_add(CGroupSubPathFull,
+    case subtree_control_add(subdirectory(CGroupPathFull, Path),
                              lists:join($ , ControlAdded),
                              Path) of
         ok ->
-            Result = update_parameters(CGroupParameters, CGroupPathFull),
-            case subtree_control_remove(CGroupSubPathFull,
-                                        lists:join($ , ControlRemoved),
-                                        Path) of
-                ok ->
-                    Result;
-                {error, _} = Error ->
-                    if
-                        Result =:= ok ->
-                            Error;
-                        true ->
-                            Result
-                    end
-            end;
+            update_parameters(CGroupParameters, CGroupPathFull);
         {error, _} = Error ->
             Error
     end.
@@ -606,9 +616,6 @@ update_parameters([{SubsystemParameter, Value} | CGroupParameters],
             {error, {subsystem_parameter, Status, Output}}
     end.
 
-update_pids(OSPids, _, CGroupPathFull) ->
-    update_pids(OSPids, CGroupPathFull).
-
 update_pids([], _) ->
     ok;
 update_pids([OSPid | OSPids], CGroupPathFull) ->
@@ -668,17 +675,6 @@ subtree_control_add(CGroupSubPathFull, Value, Path) ->
             Error
     end.
 
-subtree_control_remove(Path, Value, Path) ->
-    subtree_control_set(Path, Value);
-subtree_control_remove(CGroupSubPathFull, Value, Path) ->
-    case subtree_control_set(CGroupSubPathFull, Value) of
-        ok ->
-            subtree_control_remove(subdirectory(CGroupSubPathFull),
-                                   Value, Path);
-        {error, _} = Error ->
-            Error
-    end.
-
 subtree_control_set(CGroupSubPathFull, Value) ->
     case shell("echo \"~s\" > \"~scgroup.subtree_control\"",
                [Value, CGroupSubPathFull]) of
@@ -688,6 +684,11 @@ subtree_control_set(CGroupSubPathFull, Value) ->
             {error, {subtree_control, Status, Output}}
     end.
 
+subdirectory(Path, Path) ->
+    Path;
+subdirectory(Path, _) ->
+    subdirectory(Path).
+
 subdirectory(Path) ->
     case lists:reverse(Path) of
         [$/ | PathRev] ->
