@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2013-2019 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @copyright 2013-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_tcp).
@@ -384,9 +384,11 @@ cloudi_service_handle_info({inet_async, Listener, Acceptor, {ok, Socket}},
             end, [link]),
             case gen_tcp:controlling_process(Socket, SocketPid) of
                 ok = InitSuccess ->
-                    SocketPid ! {init, InitSuccess};
+                    SocketPid ! {init, InitSuccess},
+                    ok;
                 {error, _} = InitError ->
-                    SocketPid ! {init, InitError}
+                    SocketPid ! {init, InitError},
+                    ok
             end,
             ConnectionCount + 1;
         {error, Reason} ->
@@ -441,7 +443,7 @@ socket_loop_init_set(true, Scope,
                            DestinationSetRequestInfo, <<"SET">>,
                            TimeoutSync, Self) of
         {ok, ResponseInfo, _Response} ->
-            KeyValues = cloudi_request_info:key_value_parse(ResponseInfo),
+            KeyValues = cloudi_response_info:key_value_parse(ResponseInfo),
             NewDestination = case cloudi_key_value:find(<<"service_name">>,
                                                         KeyValues) of
                 error ->
@@ -757,6 +759,10 @@ socket_loop(#state_socket{
             socket_loop_terminate(normal, State)
     end.
 
+-spec socket_loop_terminate(Reason :: atom(),
+                            #state_socket{}) ->
+    no_return().
+
 socket_loop_terminate(Reason,
                       #state_socket{
                           socket = Socket,
@@ -768,9 +774,10 @@ socket_loop_terminate(Reason,
                           debug_level = DebugLevel}) ->
     if
         is_list(DestinationDisconnect) ->
-            send_async_minimal(Dispatcher, DestinationDisconnect,
-                               RequestInfo, <<"DISCONNECT">>,
-                               TimeoutAsync, self());
+            _ = send_async_minimal(Dispatcher, DestinationDisconnect,
+                                   RequestInfo, <<"DISCONNECT">>,
+                                   TimeoutAsync, self()),
+            ok;
         true ->
             ok
     end,
@@ -786,7 +793,7 @@ socket_loop_terminate(Reason,
     erlang:exit(Reason).
 
 socket_terminate_check(Socket, Info) ->
-    KeyValues = cloudi_request_info:key_value_parse(Info),
+    KeyValues = cloudi_response_info:key_value_parse(Info),
     case cloudi_key_value:find(<<"connection">>, KeyValues) of
         {ok, <<"close">>} ->
             self() ! {tcp_closed, Socket},
