@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,17 +28,23 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2017-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_quickrand_cache).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([float/0,
          float/1,
+         floatL/0,
+         floatL/1,
+         floatM/0,
+         floatM/1,
+         floatR/0,
+         floatR/1,
          init/0,
          init/1,
          new/0,
@@ -75,8 +81,13 @@
     float().
 
 float() ->
-    <<I:53/unsigned-integer, Bit:1, _:2>> = rand_bytes(7),
-    (I + Bit) * ?DBL_EPSILON_DIV2.
+    <<Bit:1, I:53/unsigned-integer, _:2>> = rand_bytes(7),
+    if
+        Bit == 1, I == 0 ->
+            1.0;
+        true ->
+            I * ?DBL_EPSILON_DIV2
+    end.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -88,8 +99,102 @@ float() ->
     {float(), state()}.
 
 float(State) ->
-    {<<I:53/unsigned-integer, Bit:1, _:2>>, NewState} = rand_bytes(7, State),
-    {(I + Bit) * ?DBL_EPSILON_DIV2, NewState}.
+    {<<Bit:1, I:53/unsigned-integer, _:2>>, NewState} = rand_bytes(7, State),
+    Value = if
+        Bit == 1, I == 0 ->
+            1.0;
+        true ->
+            I * ?DBL_EPSILON_DIV2
+    end,
+    {Value, NewState}.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Process dictionary cache version of cloudi_x_quickrand:strong_floatL/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatL() ->
+    float().
+
+floatL() ->
+    <<I:53/unsigned-integer, _:3>> = rand_bytes(7),
+    I * ?DBL_EPSILON_DIV2.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===State cache version of cloudi_x_quickrand:strong_floatL/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatL(State :: state()) ->
+    {float(), state()}.
+
+floatL(State) ->
+    {<<I:53/unsigned-integer, _:3>>, NewState} = rand_bytes(7, State),
+    {I * ?DBL_EPSILON_DIV2, NewState}.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Process dictionary cache version of cloudi_x_quickrand:strong_floatM/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatM() ->
+    float().
+
+floatM() ->
+    <<I:53/unsigned-integer, _:3>> = rand_bytes(7),
+    if
+        I == 0 ->
+            floatM();
+        true ->
+            I * ?DBL_EPSILON_DIV2
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===State cache version of cloudi_x_quickrand:strong_floatM/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatM(State :: state()) ->
+    {float(), state()}.
+
+floatM(State) ->
+    {<<I:53/unsigned-integer, _:3>>, NewState} = rand_bytes(7, State),
+    if
+        I == 0 ->
+            floatM(NewState);
+        true ->
+            {I * ?DBL_EPSILON_DIV2, NewState}
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Process dictionary cache version of cloudi_x_quickrand:strong_floatR/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatR() ->
+    float().
+
+floatR() ->
+    <<I:53/unsigned-integer, _:3>> = rand_bytes(7),
+    (I + 1) * ?DBL_EPSILON_DIV2.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===State cache version of cloudi_x_quickrand:strong_floatR/0.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec floatR(State :: state()) ->
+    {float(), state()}.
+
+floatR(State) ->
+    {<<I:53/unsigned-integer, _:3>>, NewState} = rand_bytes(7, State),
+    {(I + 1) * ?DBL_EPSILON_DIV2, NewState}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -296,7 +401,15 @@ option(Key, Options) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-process_dictionary_basic_test() ->
+-include("cloudi_x_quickrand_test.hrl").
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
+        {"process dictionary tests", ?_assertOk(t_process_dictionary())},
+        {"state tests", ?_assertOk(t_state())}
+    ]}.
+
+t_process_dictionary() ->
     ok = init([{cache_size, 8}]),
     Random0 = rand_bytes(2),
     2 = byte_size(Random0),
@@ -310,7 +423,7 @@ process_dictionary_basic_test() ->
     1023 = byte_size(Random4),
     ok.
 
-state_basic_test() ->
+t_state() ->
     State0 = new([{cache_size, 8}]),
     {Random0, State1} = rand_bytes(2, State0),
     2 = byte_size(Random0),
