@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -62,7 +62,7 @@ sub run
         }
         else
         {
-            print "$e";
+            print STDERR "$e";
         }
     }
     print "terminate count perl\n";
@@ -71,7 +71,7 @@ sub run
 sub assert
 {
     my ($test) = @_;
-    CloudI::API->assert($test);
+    CloudI::API::assert($test);
 }
 
 sub _request
@@ -89,13 +89,15 @@ sub _request
     }
     print "count == $self->{count} perl\n";
     my $response = "$self->{count}";
+    my $response_info = CloudI::API::info_key_value_new({});
     $self->{api}->return_($request_type, $name, $pattern,
-                          '', $response, $timeout, $trans_id, $pid);
+                          $response_info, $response,
+                          $timeout, $trans_id, $pid);
 }
 
 {
     assert($use_threads);
-    my $thread_count = CloudI::API->thread_count();
+    my $thread_count = CloudI::API::thread_count();
     my @threads = ();
     for my $i (0 .. ($thread_count - 1))
     {
