@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -34,6 +34,7 @@ import System.Exit (ExitCode(ExitFailure),exitWith)
 import qualified Control.Concurrent as Concurrent
 import qualified Data.ByteString as ByteString
 import qualified Data.ByteString.Char8 as Char8
+import qualified Data.Map.Strict as Map
 import qualified Foreign.CloudI as CloudI
 import qualified System.IO as SysIO
 type ByteString = ByteString.ByteString
@@ -53,13 +54,15 @@ request_ _ _ _ _ _ _ _ _ _ state@State{count_value = count0} api = do
         count_str = show count1
     putStrLn $ "count == " ++ count_str ++ " haskell"
     let response = Char8.pack count_str
-    return $ CloudI.Response (response, state{count_value = count1}, api)
+        responseInfo = CloudI.infoKeyValueNew Map.empty Nothing
+    return $ CloudI.ResponseInfo (responseInfo, response,
+                                  state{count_value = count1}, api)
 
 task :: Int -> IO ()
 task threadIndex = do
     let prerr = SysIO.hPutStrLn SysIO.stderr
         prout = putStrLn
-    apiValue <- CloudI.api threadIndex State{count_value = 0}
+    apiValue <- CloudI.api threadIndex State{count_value = 0} Nothing
     case apiValue of
         Left err ->
             prerr err
